/* test2.c - mm test 2 */
/*
 *  PUPA  --  Preliminary Universal Programming Architecture for GRUB
 *  Copyright (C) 2002 Yoshinori K. Okuji <okuji@enbug.org>
 *
 *  PUPA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with PUPA; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "mm.h"
#include <stdlib.h>
#include <assert.h>
#include <stdio.h>

#define DUMP() pupa_mm_dump (__LINE__)

#define REGION_SIZE	0x10000
#define TABLE_SIZE	509
#define NUM_TESTS	10000
#define MAX_SIZE	0x101
#define RANDOM_SEED	1234

static void *table[TABLE_SIZE];

int
main (void)
{
  void *r;
  unsigned i;
  void *p;

  r = malloc (REGION_SIZE);
  if (! r)
    {
      perror ("malloc");
      return 1;
    }
  
  pupa_mm_init_region (r, REGION_SIZE);
  DUMP ();

  for (i = 0; i < TABLE_SIZE; i++)
    table[i] = 0;

  srand (RANDOM_SEED);
  
  for (i = 0; i < NUM_TESTS; i++)
    {
      int n = rand () % TABLE_SIZE;

      if (table[n])
	{
	  printf ("freeing %p...\n", table[n]);
	  pupa_free (table[n]);
	  table[n] = 0;
	  DUMP ();
	}
      else
	{
	  unsigned size = rand () % MAX_SIZE;
	  
	  printf ("mallocing %u...\n", size);
	  table[n] = pupa_malloc (size);
	  if (table[n])
	    {
	      memset (table[n], 0xab, size);
	    }
	  DUMP ();
	}
    }
  
  free (r);

  return 0;
}
